@@ -1,7 +1,9 @@
 from collections import defaultdict
+from packaging import version
 import yaml
 import re
 
+from gen3utils.manifest.manifest_validator import get_manifest_version
 from gen3utils.etl.dd_utils import init_dictionary
 from gen3utils.errors import MappingError, PropertiesError, PathError, FieldError
 
@@ -12,9 +14,12 @@ class Prop:
 
 
 class Index:
-    def __init__(self, name):
+    def __init__(self, name, underscore=False):
         self.name = name
-        id_name = "{}_id".format(name)
+        if underscore:
+            id_name = "_{}_id".format(name)
+        else:
+            id_name = "{}_id".format(name)
         self.props = {id_name: Prop(id_name)}
 
 
@@ -163,11 +168,14 @@ def validate_name_src(json_obj, path, recorded_errors, nodes_with_props):
         )
     else:
         path_items = path.split(".")
-        if fn != "count" and src not in nodes_with_props.get(path_items[-1], []):
+        builtin_fields = ["source_node"]
+        valid_fields = nodes_with_props.get(path_items[-1], [])
+        valid_fields.extend(builtin_fields)
+        if fn != "count" and src not in valid_fields:
             recorded_errors.append(
                 FieldError(
-                    'src field "{}" (declared in "{}") is not found in given dictionary.'.format(
-                        src, json_obj
+                    'src field "{}" (declared in "{}" "{}") is not found in given dictionary.'.format(
+                        src, path, json_obj
                     )
                 )
             )
@@ -277,11 +285,11 @@ def check_mapping_format(mappings, recorded_errors):
     return recorded_errors
 
 
-def check_mapping_constraints(mappings, model, recorded_errors):
+def check_mapping_constraints(mappings, model, recorded_errors, underscore):
     labels_to_back_refs, nodes_with_props, categories_to_labels = get_all_nodes(model)
     indices = {}
     for m in mappings.get("mappings"):
-        index = Index(m.get("doc_type"))
+        index = Index(m.get("doc_type"), underscore)
         indices[index.name] = index
         category = m.get("category")
         if category is not None:
@@ -309,13 +317,37 @@ def check_mapping_constraints(mappings, model, recorded_errors):
     return recorded_errors
 
 
-def validate_mapping(dictionary_url, mapping_file):
+def is_release_tag(parsed_version):
+    """
+    Args:
+        parsed_version: releases.version
+    Returns:
+        bool: True if version's major number is >= 2019. This is true of
+        our current monthly release versions (e.g. 2020.05, 2019.11) but not
+        true of our standard semver versions (e.g. 2.33.0)
+    """
+    return parsed_version >= version.parse("2019.0")
+
+
+def validate_mapping(dictionary_url, mapping_file, manifest):
     dictionary, model = init_dictionary(dictionary_url)
     with open(mapping_file) as f:
         mappings = yaml.safe_load(f)
 
+    # If using tube >= 0.4.0 or >= 2020.10, {doc_type}_id fields have a prefixed underscore.
+    # https://github.com/uc-cdis/tube/releases/tag/0.4.0
+    tube_version = get_manifest_version(
+        manifest["versions"], "tube", release_tag_are_branches=False
+    )
+    underscore = False
+    if tube_version is not None and type(tube_version) != str:  # str if branch
+        if not is_release_tag(tube_version) and tube_version >= version.parse("0.4.0"):
+            underscore = True
+        elif tube_version >= version.parse("2020.10"):
+            underscore = True
+
     recorded_errors = check_mapping_format(mappings, [])
     if len(recorded_errors) > 0:
         return recorded_errors
 
-    return check_mapping_constraints(mappings, model, recorded_errors)
+    return check_mapping_constraints(mappings, model, recorded_errors, underscore)
